###############################################################################
#
# Copyright (c) 2023 HERE Europe B.V.
#
# SPDX-License-Identifier: MIT
# License-Filename: LICENSE
#
###############################################################################

from flexpolyline import iter_decode, encode

from .decoding import iter_decode_legacy
from .encoding import encode_legacy

from typing import Sequence, Union, Tuple


def decode_legacy(encoded: str) -> Sequence[Union[Tuple[str, str], Tuple[str, str, str]]]:
    """Return a list of coordinates. The number of coordinates are 2 or 3
    depending on the polyline content.
    The third dimension is specifying the corridor width for upcoming segments (CW, HW or DW).
    """
    return list(iter_decode_legacy(encoded))


def convert_flex_to_legacy(encoded: str) -> str:
    """Return a HERE (legacy) polyline out of a HERE flexible polyline encoded string. The third dimension is ignored and the width is left empty.
    """
    iter_decoded = iter_decode(encoded)
    reencoded = encode_legacy(iter_decoded)
    return reencoded


def convert_legacy_to_flex(encoded: str) -> str:
    """
    Return a HERE flexible polyline with no third dimension out of a HERE (legacy) polyline encoded string.
    The HERE Polyline third dimension is ignored.
    """
    decoded = decode_legacy(encoded)
    reencoded = encode(decoded)
    return reencoded



###############################################################################
#
# Copyright (c) 2023 HERE Europe B.V.
#
# SPDX-License-Identifier: MIT
# License-Filename: LICENSE
#
###############################################################################

from flexpolyline.encoding import encode_scaled_value
from typing import Iterable, Callable, Dict

__all__ = ['encode_legacy', 'PBAPI_WIDTHS']


PBAPI_WIDTHS: Dict[str, str] = {"DW": ".D", "CW": ".C", "HW": ".H"}


def encode_legacy(coordinates: Iterable, precision=5) -> str:
    """Encode a sequence of lat,lng or lat,lng(,{width_type}).
    `width_type`: Additional 3rd dimension precises the type for upcoming segments (CW, HW or DW).
    `precision`: how many decimal digits of precision to store the latitude and longitude.
    """
    multiplier_degree = 10 ** precision

    last_lat = last_lng = 0

    res = []
    appender: Callable[[str], None] = res.append

    for location in coordinates:
        lat = int(round(location[0] * multiplier_degree))
        encode_scaled_value(lat - last_lat, appender)
        last_lat = lat

        lng = int(round(location[1] * multiplier_degree))
        encode_scaled_value(lng - last_lng, appender)
        last_lng = lng

        try:
            if len(location) > 2:
                appender(PBAPI_WIDTHS[location[2]])
        except KeyError:
            raise ValueError(f"third_dim not one of: {', '.join(PBAPI_WIDTHS.keys())}")

    return ''.join(res)
